import datetime
import uuid

from sqlalchemy import Column, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base


class Users(Base):
    __tablename__ = "users"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    email = Column(TEXT(), unique=True, nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)


class Credentials(Base):
    __tablename__ = "credentials"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    username = Column(TEXT(), unique=True, nullable=False)
    password = Column(BYTEA(), nullable=False)
    last_seen = Column(TIMESTAMP())
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )

    user = relationship(Users, backref="credentials", uselist=False)


class Owners(Base):
    __tablename__ = "owners"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    surname = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="owners", uselist=False)



import os
import time
import json
import datetime
import logging
from logging.handlers import RotatingFileHandler
import pprint

from gaea.config import CONFIG


# class ContextFilter(logging.Filter):
#     def filter(self, record):
#         if flask.has_request_context():
#             # when logging out, the user_id is already set
#             if not hasattr(record, "user_id"):
#                 record.user_id = flask.session.get("user_id") or 0

#             record.request_form = {
#                 key: item
#                 for key, item in flask.request.form.items()
#                 if key not in ("password", "pwd")
#             }

#             record.request_id = flask.g.request_uuid
#             record.request_path = flask.request.path
#             record.request_method = flask.request.method
#             record.request_user_agent = flask.request.user_agent
#             record.request_ip_address = flask.request.remote_addr
#         return True


class CustomLogger(logging.Logger):
    """Custom logger"""

    def debug(self, msg, *args, **kwargs):
        self._log(logging.DEBUG, msg, args, extra=kwargs)

    def info(self, msg, *args, **kwargs):
        self._log(logging.INFO, msg, args, extra=kwargs)

    def warning(self, msg, *args, **kwargs):
        self._log(logging.WARNING, msg, args, extra=kwargs)

    def critical(self, msg, *args, **kwargs):
        self._log(logging.CRITICAL, msg, args, extra=kwargs)

    def error(self, msg, *args, exc_info=None, **kwargs):
        self._log(logging.ERROR, msg, args, exc_info=exc_info, extra=kwargs)

    def exception(self, msg, *args, **kwargs):
        self._log(logging.ERROR, msg, args, exc_info=True, extra=kwargs)


EXCLUDED_FIELDS = (
    "msg",
    "asctime",
    "args",
    "filename",
    "module",
    "created",
    "msecs",
    "relativeCreated",
    "thread",
    "threadName",
    "processName",
    "process",
    "levelno",
)


class BaseFormatter(logging.Formatter):
    converter = time.gmtime

    def formatTime(self, record, datefmt):
        return datetime.datetime.fromtimestamp(
            record.created, tz=datetime.timezone.utc
        ).isoformat()

    def format(self, record):
        record.message = record.getMessage()
        setattr(record, "timestamp", self.formatTime(record, self.datefmt))

        return {
            key: item
            for key, item in record.__dict__.items()
            if item is not None and key not in EXCLUDED_FIELDS
        }


class JSONFormatter(BaseFormatter):
    def format(self, record):
        log_data = super().format(record)
        return json.dumps(log_data, default=str)


class PrettyJSONFormatter(BaseFormatter):
    def format(self, record):
        log_data = super().format(record)
        return pprint.pformat(log_data)


LOG_LEVEL = CONFIG.get("LOG_LEVEL", logging.INFO)
logger = CustomLogger(CONFIG.SERVICE_NAME)
logger.setLevel(LOG_LEVEL)

DATE_FORMAT = "%Y-%m-%dT%H:%M:%S"

json_formatter = JSONFormatter(datefmt=DATE_FORMAT)
pretty_json_formatter = PrettyJSONFormatter(datefmt=DATE_FORMAT)
normal_formatter = logging.Formatter(
    fmt="{asctime} | {levelname:8s} | {message}", datefmt=DATE_FORMAT, style="{",
)


if CONFIG.get("LOG_TO_FILE"):
    log_file_name = f"{CONFIG.SERVICE_NAME}.log"
    log_path = os.path.join(CONFIG.LOGS_FOLDER_NAME, log_file_name)
    file_handler = RotatingFileHandler(log_path, "a", 1_000_000, 100)
    file_handler.setLevel(LOG_LEVEL)
    file_handler.setFormatter(json_formatter)
    logger.addHandler(file_handler)


stream_handler = logging.StreamHandler()
stream_handler.setLevel(LOG_LEVEL)

if CONFIG.get("LOG_FORMAT") == "minimize":
    stream_log_format = normal_formatter
else:
    stream_log_format = pretty_json_formatter

stream_handler.setFormatter(stream_log_format)
logger.addHandler(stream_handler)


# add context within flask.request context
# def init_logging_filter():
#     context_filter = ContextFilter()
#     logger.addFilter(context_filter)



import uuid

from sqlalchemy import MetaData
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.types import TypeDecorator


convention = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}

Base = declarative_base()
Base.metadata = MetaData(naming_convention=convention)



import uuid
import datetime

from gaea.models import (
    Users,
    Credentials,
    Owners,
    Swarms,
    SwarmHealthStatuses,
    Hives,
    HiveConditions,
    Apiaries,
    ApiaryStatuses,
    HoneyTypes,
    Comments,
    CommentTypes,
    Events,
    EventStatuses,
    EventTypes,
)


def generate_uuid(nb):
    return tuple(uuid.uuid4() for _ in range(nb))


USER_EMAIL = "coucou@cou.com"
USER_USERNAME = "coucou"
USER_PASSWORD = "password"
SWARM_HEALTH_STATUS_GOOD = "Good"
SWARM_HEALTH_STATUS_BAD = "Bad"
HIVES_CONDITION_NEW = "New"
HIVES_CONDITION_OLD = "Old"
APIARY_STATUS_ACTIVE = "Active"
APIARY_STATUS_INACTIVE = "Active"
COMMENT_TYPE_USER = "User"
COMMENT_TYPE_SYSTEM = "System"
EVENT_STATUS_PLANIFIED = "Planified"
EVENT_STATUS_DONE = "Done"
EVENT_TYPE_BURN_BEES = "Burn them all"
EVENT_TYPE_WATER_BEEHOUSE = "Water it"
EVENT_TYPE_FIND_NEW_BEEHOUSE = "Find new beehouse"
HONEY_TYPE_CHESTNUT = "Chestnut"
HONEY_TYPE_ALL_FLOWERS = "All flowers"
HONEY_TYPE_SUNFLOWER = "Sunflower"

IDS = {
    "Users": generate_uuid(1),
    "Credentials": generate_uuid(1),
    "Owners": generate_uuid(2),
    "Swarms": generate_uuid(3),
    "Hives": generate_uuid(3),
    "Apiaries": generate_uuid(3),
    "Swarm_health_statuses": generate_uuid(2),
    "Apiary_statuses": generate_uuid(2),
    "Honey_types": generate_uuid(3),
    "Hive_conditions": generate_uuid(2),
    "Comment_types": generate_uuid(2),
    "Event_types": generate_uuid(3),
    "Event_statuses": generate_uuid(2),
    "Events": generate_uuid(2),
}

DATASET = (
    Users(id=IDS["Users"][0], email=USER_EMAIL),
    Credentials(
        id=uuid.uuid4(),
        username=USER_USERNAME,
        password=b"password",
        user_id=IDS["Users"][0],
    ),
    Owners(id=IDS["Owners"][0], surname="owner1", user_id=IDS["Users"][0]),
    Owners(id=IDS["Owners"][1], surname="owner2", user_id=IDS["Users"][0]),
    SwarmHealthStatuses(
        id=IDS["Swarm_health_statuses"][0],
        name=SWARM_HEALTH_STATUS_GOOD,
        user_id=IDS["Users"][0],
    ),
    SwarmHealthStatuses(
        id=IDS["Swarm_health_statuses"][1],
        name=SWARM_HEALTH_STATUS_BAD,
        user_id=IDS["Users"][0],
    ),
    Swarms(
        id=IDS["Swarms"][0],
        health_status_id=IDS["Swarm_health_statuses"][0],
        user_id=IDS["Users"][0],
    ),
    Swarms(
        id=IDS["Swarms"][1],
        health_status_id=IDS["Swarm_health_statuses"][0],
        user_id=IDS["Users"][0],
    ),
    Swarms(
        id=IDS["Swarms"][2],
        health_status_id=IDS["Swarm_health_statuses"][1],
        user_id=IDS["Users"][0],
    ),
    ApiaryStatuses(
        id=IDS["Apiary_statuses"][0], name=APIARY_STATUS_ACTIVE, user_id=IDS["Users"][0]
    ),
    ApiaryStatuses(
        id=IDS["Apiary_statuses"][1],
        name=APIARY_STATUS_INACTIVE,
        user_id=IDS["Users"][0],
    ),
    HoneyTypes(
        id=IDS["Honey_types"][0], name=HONEY_TYPE_ALL_FLOWERS, user_id=IDS["Users"][0]
    ),
    HoneyTypes(
        id=IDS["Honey_types"][1], name=HONEY_TYPE_CHESTNUT, user_id=IDS["Users"][0]
    ),
    HoneyTypes(
        id=IDS["Honey_types"][2], name=HONEY_TYPE_SUNFLOWER, user_id=IDS["Users"][0]
    ),
    Apiaries(
        id=IDS["Apiaries"][0],
        name="apiary1",
        location="location1",
        user_id=IDS["Users"][0],
        status_id=IDS["Apiary_statuses"][0],
        honey_type_id=IDS["Honey_types"][0],
    ),
    Apiaries(
        id=IDS["Apiaries"][1],
        name="apiary2",
        location="location2",
        user_id=IDS["Users"][0],
        status_id=IDS["Apiary_statuses"][0],
        honey_type_id=IDS["Honey_types"][1],
    ),
    Apiaries(
        id=IDS["Apiaries"][2],
        name="apiary3",
        location="location3",
        user_id=IDS["Users"][0],
        status_id=IDS["Apiary_statuses"][1],
        honey_type_id=IDS["Honey_types"][2],
    ),
    HiveConditions(
        id=IDS["Hive_conditions"][0], name=HIVES_CONDITION_NEW, user_id=IDS["Users"][0]
    ),
    HiveConditions(
        id=IDS["Hive_conditions"][1], name=HIVES_CONDITION_OLD, user_id=IDS["Users"][0]
    ),
    Hives(
        id=IDS["Hives"][0],
        name="hive1",
        user_id=IDS["Users"][0],
        condition_id=IDS["Hive_conditions"][0],
        owner_id=IDS["Owners"][0],
        swarm_id=IDS["Swarms"][0],
        apiary_id=IDS["Apiaries"][0],
    ),
    Hives(
        id=IDS["Hives"][1],
        name="hive2",
        user_id=IDS["Users"][0],
        condition_id=IDS["Hive_conditions"][0],
        owner_id=IDS["Owners"][1],
        swarm_id=IDS["Swarms"][1],
        apiary_id=IDS["Apiaries"][1],
    ),
    Hives(
        id=IDS["Hives"][2],
        name="hive3",
        user_id=IDS["Users"][0],
        condition_id=IDS["Hive_conditions"][1],
        owner_id=IDS["Owners"][0],
        swarm_id=IDS["Swarms"][2],
        apiary_id=IDS["Apiaries"][2],
    ),
    EventTypes(
        id=IDS["Event_types"][0], name=EVENT_TYPE_BURN_BEES, user_id=IDS["Users"][0]
    ),
    EventTypes(
        id=IDS["Event_types"][1],
        name=EVENT_TYPE_WATER_BEEHOUSE,
        user_id=IDS["Users"][0],
    ),
    EventTypes(
        id=IDS["Event_types"][2],
        name=EVENT_TYPE_FIND_NEW_BEEHOUSE,
        user_id=IDS["Users"][0],
    ),
    EventStatuses(
        id=IDS["Event_statuses"][0],
        name=EVENT_STATUS_PLANIFIED,
        user_id=IDS["Users"][0],
    ),
    EventStatuses(
        id=IDS["Event_statuses"][1], name=EVENT_STATUS_DONE, user_id=IDS["Users"][0]
    ),
    Events(
        id=IDS["Events"][0],
        title="title1",
        description="description1",
        due_date=datetime.datetime.utcnow(),
        user_id=IDS["Users"][0],
        type_id=IDS["Event_types"][0],
        status_id=IDS["Event_statuses"][1],
        hive_id=IDS["Hives"][0],
    ),
    Events(
        id=IDS["Events"][1],
        title="title2",
        description="description2",
        due_date=datetime.datetime.utcnow() + datetime.timedelta(days=1),
        user_id=IDS["Users"][0],
        type_id=IDS["Event_types"][1],
        status_id=IDS["Event_statuses"][0],
        hive_id=IDS["Hives"][0],
    ),
    CommentTypes(id=IDS["Comment_types"][0], name=COMMENT_TYPE_USER),
    CommentTypes(id=IDS["Comment_types"][1], name=COMMENT_TYPE_SYSTEM),
    Comments(
        id=uuid.uuid4(),
        comment="comment1",
        date=datetime.datetime.utcnow(),
        user_id=IDS["Users"][0],
        type_id=IDS["Comment_types"][0],
        swarm_id=IDS["Swarms"][0],
        hive_id=IDS["Hives"][0],
        apiary_id=IDS["Apiaries"][0],
    ),
    Comments(
        id=uuid.uuid4(),
        comment="comment2",
        date=datetime.datetime.utcnow(),
        user_id=IDS["Users"][0],
        type_id=IDS["Comment_types"][1],
        apiary_id=IDS["Apiaries"][2],
        event_id=IDS["Events"][0],
    ),
    Comments(
        id=uuid.uuid4(),
        comment="comment3",
        date=datetime.datetime.utcnow(),
        user_id=IDS["Users"][0],
        type_id=IDS["Comment_types"][0],
        swarm_id=IDS["Swarms"][1],
        hive_id=IDS["Hives"][1],
        apiary_id=IDS["Apiaries"][1],
    ),
    Comments(
        id=uuid.uuid4(),
        comment="comment4",
        date=datetime.datetime.utcnow(),
        user_id=IDS["Users"][0],
        type_id=IDS["Comment_types"][0],
        swarm_id=IDS["Swarms"][0],
        hive_id=IDS["Hives"][0],
        apiary_id=IDS["Apiaries"][0],
    ),
)



import logging

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session

from gaea.config import CONFIG
from gaea.helpers.metaclass import Box
from gaea.errors import AlreadyInitialized


def get_engine(url):
    echo_query = CONFIG.LOG_LEVEL <= logging.DEBUG
    engine = create_engine(url, echo=echo_query, pool_size=5, pool_recycle=3600)
    return engine


def get_database_url(dbname=None):
    user = CONFIG.DATABASE_USER
    pswd = CONFIG.DATABASE_PASSWORD
    host = CONFIG.DATABASE_HOST
    port = CONFIG.DATABASE_PORT
    dbnm = dbname or CONFIG.DATABASE_DATABASE

    if any(val is None for val in (user, pswd, host, port, dbnm)):
        raise ValueError("Database configuration is missing")

    return f"postgresql+psycopg2://{user}:{pswd}@{host}:{port}/{dbnm}"


class db:
    def __init__(self, url=None):
        self.url = url or get_database_url()
        self.engine = get_engine(self.url)
        self.session_factory = sessionmaker(autoflush=False, bind=self.engine)
        self._sessions = []

    def get_session(self):
        return self.session_factory()

    def clear(self):
        self.engine.dispose()

    def __enter__(self):
        session = self.get_session()
        self._sessions.append(session)
        return session

    def __exit__(self, tp, vl, tb):
        session = self._sessions.pop(-1)
        session.rollback()
        session.close()



import fastapi
from fastapi.middleware.cors import CORSMiddleware


class AppClient:
    def __init__(self, router, middleware=None):
        self.app = fastapi.FastAPI()
        self.app.include_router(router)

        self.middleware = middleware

    def _add_middleware(self):
        if self.middleware:
            self.app.middleware("http")(self.middleware)

    def add_cors_middleware(
        self,
        allow_origins=None,
        allow_methods=None,
        allow_headers=None,
        allow_credentials=False,
    ):
        self.app.add_middleware(
            CORSMiddleware,
            allow_origins=allow_origins or ["*"],
            allow_methods=allow_methods or ["*"],
            allow_headers=allow_headers or ["*"],
            allow_credentials=allow_credentials,
        )

    def get_app(self):
        self._add_middleware()
        return self.app



import datetime
import uuid

from sqlalchemy import Column, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users


class ApiaryStatuses(Base):
    __tablename__ = "apiary_statuses"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT())
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="apiary_statuses")


class HoneyTypes(Base):
    __tablename__ = "honey_types"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT())
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="honey_types")


class Apiaries(Base):
    __tablename__ = "apiaries"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)
    location = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)
    status_id = Column(
        UUID(as_uuid=True), ForeignKey(ApiaryStatuses.id), nullable=False
    )
    honey_type_id = Column(
        UUID(as_uuid=True), ForeignKey(HoneyTypes.id), nullable=False
    )

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="apiaries")
    status = relationship(ApiaryStatuses)
    honey_type = relationship(HoneyTypes)



import contextlib
import uuid

from gaea.database.main import db, get_database_url
from gaea.models.base import Base


@contextlib.contextmanager
def get_temporary_database():
    db_client = db()
    session = db_client.get_session()
    conn = session.connection()

    dbname = "test_" + str(uuid.uuid4()).replace("-", "")
    conn.execute("commit")
    conn.execute(f"CREATE DATABASE {dbname}")
    conn.execute("commit")

    yield get_database_url(dbname)

    conn.execute(f"DROP DATABASE {dbname}")
    conn.execute("commit")

    session.close()
    conn.close()
    db_client.clear()



import datetime
import uuid

from sqlalchemy import Column, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users


class SwarmHealthStatuses(Base):
    __tablename__ = "swarm_health_statuses"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT())
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="swarm_healths")


class Swarms(Base):
    __tablename__ = "swarms"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    health_status_id = Column(
        UUID(as_uuid=True), ForeignKey(SwarmHealthStatuses.id), nullable=False
    )
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="swarms_healths")
    health = relationship(SwarmHealthStatuses)



import os
import redis

from common.log.logger import logger

from .messages import check_channel


class RedisClient:
    def __init__(self):
        self.client = self.get_client()
        self.pubsub = self.client.pubsub()

    def get_client(self):
        return redis.Redis(host=os.environ.get("REDIS_HOST", "localhost"))

    def publish(self, channel, message):
        if not check_channel(channel):
            logger.error(f"Unknown channel: {channel}")
        else:
            self.client.publish(channel, message)
            logger.info("Message published in redis: {channel} - {message}")

    def listen(self, channel, callback):
        self.pubsub.subscribe(channel)

        for msg in self.pubsub.listen():
            callback(msg)



import datetime
import uuid

from sqlalchemy import Column, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users
from .apiaries import Apiaries
from .hives import Hives


class EventTypes(Base):
    __tablename__ = "event_types"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="event_types")


class EventStatuses(Base):
    __tablename__ = "event_statuses"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="event_statuses")


class Events(Base):
    __tablename__ = "events"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    title = Column(TEXT(), nullable=False)
    description = Column(TEXT(), nullable=True)
    due_date = Column(TIMESTAMP(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)
    type_id = Column(UUID(as_uuid=True), ForeignKey(EventTypes.id), nullable=False)
    status_id = Column(UUID(as_uuid=True), ForeignKey(EventStatuses.id), nullable=False)
    hive_id = Column(UUID(as_uuid=True), ForeignKey(Hives.id), nullable=True)
    apiary_id = Column(UUID(as_uuid=True), ForeignKey(Apiaries.id), nullable=True)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="events")
    type = relationship(EventTypes)
    status = relationship(EventStatuses)
    hive = relationship(Hives, backref="events")
    apiary = relationship(Apiaries, backref="events")



import fastapi

from starlette.exceptions import HTTPException as StarletteHTTPException

from gaea.database import db
from gaea.log import logger


def get_session(request: fastapi.Request):
    return request.state.session


class MiddleWare:
    def __init__(self, db_client=None):
        self.db = db_client

    async def before_request(self, *args, **kwargs):
        pass

    async def after_request(self, *args, **kwargs):
        pass

    async def exception_handler(self, *args, **kwargs):
        pass

    async def __call__(self, request: fastapi.Request, call_next):
        await self.before_request()

        if self.db:
            session = self.db.get_session()
            request.state.session = session

        try:
            response = await call_next(request)

        except fastapi.HTTPException:
            await self.exception_handler()
            logger.exception("Managed exception happened")
            raise
        except Exception as exc:
            await self.exception_handler()
            logger.exception("Unexpected exception happened")
            raise fastapi.HTTPException(500, "Internal server error") from exc

        finally:

            if self.db:
                session.close()

            await self.after_request()

        return response



from gaea.errors import NotInitialized, AlreadyInitialized


class SingletonMeta(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            instance = super().__call__(*args, **kwargs)
            cls.instances[cls] = instance
        return cls._instances[cls]


class Box:
    def __init__(self):
        self.store = None

    def __get__(self, obj, type=None):
        if self.store is None:
            raise NotInitialized()
        return self.store

    def __set__(self, obj, value):
        if self.store is not None:
            if not self.store == value:
                raise AlreadyInitialized()
            return
        self.store = value

    def __delete__(self, obj):
        self.store = None



import os


class Config:
    LOGS_FOLDER_NAME = "log"
    IMAGES_FOLDER_NAME = "images"
    FLASK_SESSIONS_FOLDER_NAME = "flask_session"

    PATH_PROJECT = os.getcwd()
    PATH_LOGS = os.path.join(PATH_PROJECT, LOGS_FOLDER_NAME)
    PATH_IMAGES = os.path.join(PATH_PROJECT, IMAGES_FOLDER_NAME)
    PATH_FLASK_SESSIONS = os.path.join(PATH_PROJECT, FLASK_SESSIONS_FOLDER_NAME)

    URL_PREFIX = ""

    ENV = os.environ.get("ENV", "TEST")
    IDENTIFIER = os.environ.get("IDENTIFIER", "")
    SERVICE_NAME = os.environ.get("SERVICE_NAME", "Unidentified")

    SECRET_KEY = "DEV"
    PASSWORD_REQUESTED = True

    FLASK_DEBUG = False

    DATABASE_PORT = 3306
    DATABASE_HOST = None
    DATABASE_USER = None
    DATABASE_PASSWORD = None
    DATABASE_DATABASE = "akb"

    def __init__(self):
        self.load_env_variables()
        self.create_missing_folders()
        self.validate()

    def validate(self):
        assert self.ENV in ("TEST", "PROD", "SIMU")

    @staticmethod
    def load_env_file(filename):
        for line in open(filename):
            data = line.strip().split("=")
            if len(data) != 2:
                continue
            if data[0] not in os.environ:
                os.environ[data[0]] = data[1]

    def load_env_variables(self):
        if os.path.exists(".env"):
            self.load_env_file(".env")

        identifier = f"{self.IDENTIFIER}_"

        for key, value in os.environ.items():
            if not key.startswith(identifier):
                continue

            key = key.replace(identifier, "")
            value = True if value == "true" else value
            value = False if value == "false" else value

            setattr(self, key, value)

    def create_missing_folders(self):
        paths = (self.PATH_LOGS, self.PATH_FLASK_SESSIONS)

        for path in paths:
            if not os.path.exists(path):
                os.mkdir(path)

    @property
    def DATABASE(self):
        if self.ENV not in ("SIMU", "PROD"):
            return {}

        return {
            "port": self.DATABASE_PORT,
            "host": self.DATABASE_HOST,
            "user": self.DATABASE_USER,
            "password": self.DATABASE_PASSWORD,
            "database": f"{self.IDENTIFIER}_{self.ENV}".lower(),
        }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gaea',
 'gaea.config',
 'gaea.constants',
 'gaea.database',
 'gaea.database.utils',
 'gaea.errors',
 'gaea.helpers',
 'gaea.log',
 'gaea.models',
 'gaea.models.utils',
 'gaea.redis',
 'gaea.webapp']

package_data = \
{'': ['*']}

install_requires = \
['alembic>=1.4.2,<2.0.0',
 'dynaconf>=3.0.0,<4.0.0',
 'fastapi>=0.61.0,<0.62.0',
 'psycopg2-binary>=2.8.5,<3.0.0',
 'redis>=3.5.3,<4.0.0',
 'sqlalchemy>=1.3.18,<2.0.0',
 'uvicorn>=0.11.8,<0.12.0']

setup_kwargs = {
    'name': 'gaea',
    'version': '0.1.8',
    'description': '',
    'long_description': None,
    'author': 'rarnal',
    'author_email': 'arnal.romain@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import datetime
import uuid

from sqlalchemy import Column, ForeignKey
from sqlalchemy.orm import relationship, backref
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users, Owners
from .apiaries import Apiaries
from .swarms import Swarms


class HiveConditions(Base):
    __tablename__ = "hive_conditions"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="hive_conditions")


class Hives(Base):
    __tablename__ = "hives"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)

    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)
    condition_id = Column(
        UUID(as_uuid=True), ForeignKey(HiveConditions.id), nullable=False
    )
    owner_id = Column(UUID(as_uuid=True), ForeignKey(Owners.id), nullable=True)
    swarm_id = Column(UUID(as_uuid=True), ForeignKey(Swarms.id), nullable=True)
    apiary_id = Column(UUID(as_uuid=True), ForeignKey(Apiaries.id), nullable=True)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="hives")
    owner = relationship(Owners, backref="hives")
    swarm = relationship(Swarms, backref=backref("hive", uselist=False))
    apiary = relationship(Apiaries, backref="hives")
    condition = relationship(HiveConditions, backref="hives")



from .users import Users, Owners, Credentials
from .apiaries import Apiaries, ApiaryStatuses, HoneyTypes
from .hives import Hives, HiveConditions
from .swarms import Swarms, SwarmHealthStatuses
from .events import Events, EventStatuses, EventTypes
from .comments import Comments, CommentTypes



import datetime
import uuid

from sqlalchemy import Column, ForeignKey
from sqlalchemy.orm import relationship, backref
from sqlalchemy.dialects.postgresql import TIMESTAMP, TEXT, BYTEA, UUID

from .base import Base
from .users import Users
from .swarms import SwarmHealthStatuses, Swarms
from .apiaries import Apiaries
from .hives import Hives, HiveConditions
from .events import Events


class CommentTypes(Base):
    __tablename__ = "comment_types"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = Column(TEXT(), nullable=False)

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)


class Comments(Base):
    __tablename__ = "comments"
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    comment = Column(TEXT(), nullable=False)
    date = Column(TIMESTAMP(), nullable=False)
    user_id = Column(UUID(as_uuid=True), ForeignKey(Users.id), nullable=False)
    type_id = Column(UUID(as_uuid=True), ForeignKey(CommentTypes.id), nullable=False)
    swarm_id = Column(UUID(as_uuid=True), ForeignKey(Swarms.id), nullable=True)
    hive_id = Column(UUID(as_uuid=True), ForeignKey(Hives.id), nullable=True)
    apiary_id = Column(UUID(as_uuid=True), ForeignKey(Apiaries.id), nullable=True)
    event_id = Column(UUID(as_uuid=True), ForeignKey(Events.id), nullable=True)
    swarm_health_status_id = Column(
        UUID(as_uuid=True), ForeignKey(SwarmHealthStatuses.id), nullable=True
    )
    hive_condition_id = Column(
        UUID(as_uuid=True), ForeignKey(HiveConditions.id), nullable=True
    )

    created_at = Column(TIMESTAMP(), default=datetime.datetime.utcnow)
    updated_at = Column(
        TIMESTAMP(), default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow
    )
    deleted_at = Column(TIMESTAMP(), nullable=True)

    user = relationship(Users, backref="comments")
    type = relationship(CommentTypes)
    swarm_health_status = relationship(SwarmHealthStatuses)
    hive_condition = relationship(HiveConditions)
    swarm = relationship(Swarms, backref="comments")
    hive = relationship(Hives, backref="comments")
    apiary = relationship(Apiaries, backref="comments")
    event = relationship(Events, backref=backref("comment", uselist=False))
